#!/usr/bin/perl -w

#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.


package Nmap::Group;

use strict;
use DBI;
use Nmap::Functions;
use Nmap::Logger;

use constant
{
	OPT_NORMAL	=>	0,
	OPT_NEW	=>	1,
	OPT_UPDATE	=>	2,
	OPT_DELETE => 	-1,
	RIGHTS		=>  [	'SCAN-TCP'  , 
						'SCAN-UDP' , 
						'PING-TARGET' , 
						'VERSION' , 
						'OSDETECT' , 
						'OSDETECT-AGRESSIVE' , 
						'RTT-CHANGE',
						'PARALLELISM-CHANGE',
						'ALLOW-IPV6',
						'MAIL-RESULT',
						'DENY-DOMAINS',
						'ONLY-DOMAINS',
						'ALLOW-SUBNET',
						'ALLOW-SCAN-SINGLE',
						'ALLOW-SCAN-SCHEDULED',
						'ALLOW-SCAN-PERIODIC',
						'MAX-SCANS-PER-DAY',
						'MAX-SCANS-PER-WEEK',
						'TIMING-TEMPLATES',
						'TREAT-ONLINE',
						'PORTS-RANGE',
					],
};

sub new
{
    my $this;
    my $class;
	my $nmap;
	my $groupname;
	my $opts;
    
    $class 		= 	shift;
	$nmap		=	shift;
	$groupname	=	shift;
	$opts		=	shift;


    $this = {};
    
   	return( undef ) if( ! defined( $groupname ) );

    $this->{'main'} = $nmap;

    $this->{'groupname'} = $groupname;

	if( defined $opts )
	{
    	$this->{'id'} = $groupname if( ( defined $opts->{METHOD} ) && ( $opts->{METHOD} eq 'id' ) );
	}
    
    bless( $this , $class );
}


sub create
{
	my $self = shift;
	my $query;
	my $stmt;

	$query = "INSERT INTO groups( groupname , def ) VALUES( ? , 'N' ) ";
	$stmt = $self->{'main'}->get_db->prepare( $query );
	$stmt->execute( $self->{'groupname'} ) or return -1;

	$self->save;

	return 1;
}

sub save
{
	my $self = shift;
	my $query;
	my $opt;
	my $sth;
	my $tmp;


	#Save if it's the default group or not
	$query = "UPDATE groups SET def=? WHERE id=?";
	$sth = $self->{'main'}->get_db->prepare( $query );
	nmaplog( "toto" . $self->{def} );
	$sth->execute( $self->{def} , $self->{id} ) or return -1;

	# Update rights for the group
	$tmp = RIGHTS;
	use Nmap::Logger;
	foreach $opt ( @$tmp )
	{
		nmaplog( "teste le right " . $opt );
		next if( ! defined $self->{options}->{$opt} );
		next if( ! defined $self->{options}->{$opt}{state} );
		next if( $self->{options}->{$opt}{state} == OPT_NORMAL);
		
		if( $self->{options}->{$opt}{'state'} == OPT_NEW )
		{
			if( defined $self->{options}->{$opt}{'value'} )
			{
				$query = "INSERT INTO group_rights( gid , rname , value) VALUES( ? , ? , ? )";
				$sth = $self->{'main'}->get_db->prepare( $query );
				$sth->execute( $self->{id} , $opt , $self->{options}->{$opt}{'value'} ) or return -1;
			}
			else
			{
				$query = "INSERT INTO group_rights( gid , rname ) VALUES( ? , ? )";
				$sth = $self->{'main'}->get_db->prepare( $query );
				$sth->execute( $self->{id} , $opt ) or return -1;
			}
		}
		elsif( $self->{options}->{$opt}{'state'} == OPT_UPDATE )
		{
			if( defined $self->{options}->{$opt}{'value'} )
			{
				$query = "UPDATE group_rights SET value=? WHERE gid=? AND rname=?";
				$sth = $self->{'main'}->get_db->prepare( $query );
				$sth->execute( $self->{options}->{$opt}{'value'} , $self->{id} , $opt ) or return -1;
			}
		}
		elsif( $self->{options}->{$opt}{'state'} == OPT_DELETE )
		{
			$query = "DELETE FROM group_rights WHERE gid=? AND rname=?";
			$sth = $self->{'main'}->get_db->prepare( $query );
			$sth->execute( $self->{id} , $opt ) or return -1;
			undef( $self->{options}->{$opt} );
		}
	}
	return 1;
}

sub getdb
{
	my $self;
	my $query;
	my $request;
	my $result;
	my $r;

	$self = shift;

	if( defined $self->{id} )
	{
		$query = "SELECT * FROM groups WHERE id=?";
		$request = $self->{'main'}->get_db->prepare( $query ) or return -2;
		$r = $request->execute( $self->{id} ) or return -3;
	}
	else
	{
		$query = "SELECT * FROM groups WHERE groupname=?";
		$request = $self->{'main'}->get_db->prepare( $query ) or return -2;
		$r = $request->execute( $self->{groupname} ) or return -3;
	}
	
	return 0 if( $r < 1 );

	$result = $request->fetchrow_hashref();

	foreach ( keys %{$result} )
	{
		$self->{$_} = $result->{$_};
	}

	$query = "SELECT * FROM group_rights WHERE gid=?";
	$request = $self->{'main'}->get_db->prepare( $query ) or return -2;
	$request->execute( $self->{id} ) or return -3;

	$result = $request->fetchall_hashref( 'rname' );

	foreach ( keys %{$result} )
	{
		$self->{options}->{$_}{state} = OPT_NORMAL;
		$self->{options}->{$_}{value} = $result->{$_}{value};
	}

	return 1;
}


# Return 1 is the group has the option given in parameter
# Else, returns 0
sub opt
{
	my $self;
	my $optname;

	$self = shift;
	$optname = shift;

	if( defined( $self->{options}->{$optname} ) )
	{
		return( 1 );
	}
	return( 0 );
}


# Get the value for an option
# Returns undef is no value
sub opt_value
{
	my $self;
	my $optname;

	$self = shift;
	$optname = shift;
	
	if( $self->opt( $optname )  == 1 )
	{
		return( $self->{'options'}->{$optname}{'value'} );
	}

	return( undef );
}


# Remove an option from the group
# You can call it as it :
# $group->opt_remove( "my-opt" );
# After, call $group->save
sub opt_remove
{
	my $self;
	my $optname;

	$self = shift;
	$optname = shift;

	if( $self->opt( $optname ) == 1 )
	{
		$self->{'options'}->{$optname}{'state'} = OPT_DELETE;
		return( 1 );
	}

	return( 0 );
}

# Set the value for an option
# For example : $group->setopt( "my-option" , "my-value" );
# Or : $group->setopt( "my-option" );
sub setopt
{
	my $self;
	my $optname;
	my $value;

	$self 		= shift;
	$optname 	= shift;
	$value 		= shift;

	if( defined $self->{options}->{$optname} )
	{
		$self->{options}->{$optname}{state} = OPT_UPDATE;
	}
	else
	{
		$self->{options}->{$optname}{state} = OPT_NEW;
	}

	if( defined $value )
	{
		$self->{options}->{$optname}{value} = $value;
	}
}

sub get
{
	my $self;
	my $value;
	$self = shift;
	$value = shift;
	return( Nmap::Functions::stripslashes( $self->{$value} ) );
}

sub set
{
	my $self;
	my $key;
	my $value;
	$self = shift;
	$key = shift;
	$value = shift;

	$self->{$key} = Nmap::Functions::addslashes( $value );
}




sub getall
{
	my ( $main , $tmp , $group , @ar , @result , $opts , $query , $request , $r );
	$main = shift;
	$opts = shift;

	$opts->{ORDERBY} = 'id' if( ! ( $opts->{ORDERBY} =~ /^(groupname|def)$/ ) );
	$opts->{INVERT} = 1 if( ( $opts->{INVERT} != 1 ) && ( $opts->{INVERT} != 0 ) );
	
	$query = "SELECT groupname FROM groups WHERE 1=1";
	$query .= " AND def='".$opts->{DEFAULT}."'" if( defined $opts->{DEFAULT} );
	$query .= " AND groupname LIKE '%".$opts->{GROUPNAME}."%'" if( defined $opts->{GROUPNAME} );
	$query .= " OFFSET ".$opts->{OFFSET} if( defined $opts->{OFFSET} );
	$query .= " LIMIT ".$opts->{LIMIT} if( defined $opts->{LIMIT} );
	$query .= " ORDER BY ".$opts->{ORDERBY} if( defined $opts->{ORDERBY} );
	$query .= " DESC " if( ( defined $opts->{INVERT} ) && ( $opts->{INVERT} == 1 ) );
	$query .= " LIMIT ".$opts->{LIMIT} if( defined $opts->{LIMIT} );
	$request = $main->get_db->prepare( $query ) or return( undef );
	$r = $request->execute() or return( undef );

	return undef if( $r <= 0 );

	while( @ar = $request->fetchrow_array )
	{
		$tmp = shift( @ar );
		$group = new Nmap::Group( $main , $tmp );
		next if( $group->getdb != 1 );
		push( @result , $group );
	}
	return( \@result );
}


sub delete
{
	my ( $self , $query , $stmt );
	$self = shift;
	$query = "DELETE FROM group_rights WHERE gid=?";
	$stmt = $self->{'main'}->get_db->prepare( $query ) or return -2;
	$stmt->execute( $self->{id} ) or return -3;

	$query = "DELETE FROM groups WHERE id=?";
	$stmt = $self->{'main'}->get_db->prepare( $query ) or return -2;
	$stmt->execute( $self->{id} ) or return -3;

	return( 1 );
}

1;

__END__
=head1 SYNOPSIS

Nmap::Group handle groups for nmap-cgi program

=head1 DESCRIPTION

This module is an object which handle the groups of nmap-cgi. This module is OO oriented.

=head1 METHODS

=head2 new

Make a new instance of this object. The first param is the username. You could give an gid as second argument, but int this case, the third argument should be a hash like this : { METHOD => 'id' }

=head2 create()

Create the object in the database. Returns 1 if success.

=head2 save()

Save the current objet in the database. Save options, rights, etc ...

=head2 getdb()

Get the group from the database. If a third argument was used in the new() method and specify that we get byid, it willl search the group by gid. Otherwise, it takes the group by username. Returns 1 if succeeded.

=head2 get( $attr )

Get the atribute value which has $attr name.

=head2 setopt( $optname , $value )

Set the opttion of $optname to $value.

=head2 opt_remove( $optname )

Remove the option of $optname

=head2 opt_value( $optname )

Get the value of the option $optname

=head2 getall()

Get all groups that matches with the options. The first argument is a reference to a Nmap::Main instance. The second is a hash which contains options.
Options are hashkey, options value are hashvalues.
There are the list of hashkey :

	* def : set 1 to get the default group

	* groupname : give a string to search all groups that seems have this groupname

	* OFFSET

	* LIMIT

	* ORDERBY

	* DESC

	* LIMIT


=head1 SEE ALSO

Nmap::Main , Nmap::User

=head1 AUTHOR

Julien Delange <julien at gunnm dot org>

=head1 COPYRIGHT

Julien Delange <julien at gunnm dot org>

