#!/usr/bin/perl

#Copyright (c) Nmap project and its contributor
#All rights reserved.

#Redistribution and use in source and binary forms, with or without 
#modification, are permitted provided that the following conditions 
#are met:

# 1. Redistributions of source code must retain the above copyright 
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in 
#    the documentation and/or other materials provided with the 
#    distribution.
# 3. Neither the name of Nmap nor the names of its contributors 
#    may be used to endorse or promote products derived from this 
#    software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.

package Nmap::Main;

require Exporter;
@ISA 	= qw( Exporter );
@EXPORT = qw( debug );

use strict;
use DBI;
use Nmap::Conf;

use constant
{
    CONFPATH => "/etc/nmap-cgi/nmap-cgi.conf"
};


sub debug
{
	my $str;
	$str = shift;
	open FIC , ">>/tmp/debug";
	print FIC $str;
	close FIC;
}


sub init
{
        my $self;
        my $this;
        my $dbh;
        my $class;
        my $config;
        my $db_config;
		my $driver;
		my $dbparams;
		my $dbuser;
		my $dbpass;
        
        $this = shift;
 
        #First, create the object
        $self={};
        $class = ref($this) || $this;
        bless($self , $class );


        #Then, config stuff
        $config = new Nmap::Conf( CONFPATH );
        $self->{'config'} = $config if (defined $config);


        #Finally, backend stuff
        $dbuser = $config->get_dbuser();
        $dbpass = $config->get_dbpass();
		$dbparams= "dbname=".$config->get_dbname().";host=".$config->get_dbhost().";port=".$config->get_dbport();

		$dbh = DBI->connect("DBI:Pg:$dbparams",$dbuser, $dbpass , { RaiseError => 0 } ) or  $self->{'init'} = 0;
		if( ( ! defined( $dbh ) ) || ( ! $dbh ) )
		{
			$self->{'init'} = 0;
			$self->{'db'} = $dbh;
		}
		else
		{
			$self->{'init'} = 1;
			$self->{'db'} = $dbh;
		}

        return $self;
}

sub get_config
{
	my $self = shift;
	return( $self->{'config'} );
}

sub get_db
{
	my $self = shift;
	return( $self->{db} );
}

1;

__END__

=head1 SYNOPSIS

Nmap::Main main object to handle important stuff (configuration, backend, ...)

=head1 DESCRIPTION

This module is used to init all stuff for nmap-cgi project. It creates connection to the backend, read configuration with Nmap::Conf, ...

=head1 METHODS

=head2 init()

Read the config file, create connexion to the backend, ... Returns an instance of the new Nmap::Main object. 
If an error was reported, $returnval->{'init'} will be set to 0. If the init was good, this value is set to 1.

=head2 get_config()

Returns the Nmap::Conf object related to the current Nmap::Main instance.

=head2 get_db()

Returns the DBI object releated to the current Nmap::Main instance. It can be used to perform request on the database.


=head1 EXAMPLE
	
$main = init Nmap::Main;

$conf = $main->get_conf();

$dbi = $main->get_db();

=head1 SEE ALSO

Nmap::Config 

=head1 AUTHOR

Julien Delange <julien at gunnm dot org>

=head1 COPYRIGHT

Julien Delange <julien at gunnm dot org>

