/*
Copyright (c) Nmap project and its contributors
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions 
are met:

 1. Redistributions of source code must retain the above copyright 
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in 
    the documentation and/or other materials provided with the 
    distribution.
 3. Neither the name of Nmap nor the names of its contributors 
    may be used to endorse or promote products derived from this 
    software without specific prior written permission.

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
 LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.
*/

#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <libpq-fe.h>

#include <request.h>
#include <config.h>
#include <scan.h>
#include <debug.h>
#include <db.h>

extern PGconn*		dbconn;
extern config_t*	config;

char** opt2cmdopt( const char* opt , const char* arg )
{
	char** ret;
	ret = calloc( sizeof( char* ) , 2 );

	if( strcmp( opt , "TCP-METHOD" ) == 0 )
	{
		if( strcmp( arg , "SYN" ) == 0 )
		{
			ret[0] = "-sS";
			ret[1] = NULL;
			return( ret );
		}
		if( strcmp( arg , "CON" ) == 0 )
		{
			ret[0] = "-sT";
			ret[1] = NULL;
			return( ret );
		}
		if( strcmp( arg , "NUL" ) == 0 )
		{
			ret[0] = "-sN";
			ret[1] = NULL;
			return( ret );
		}
		if( strcmp( arg , "FIN" ) == 0 )
		{
			ret[0] = "-sF";
			ret[1] = NULL;
			return( ret );
		}
		if( strcmp( arg , "XMA" ) == 0 )
		{
			ret[0] = "-sX";
			ret[1] = NULL;
			return( ret );
		}
	}
	if( strcmp( opt , "SCAN-UDP" ) == 0 )
	{
		ret[0] = "-sU";
		ret[1] = NULL;
		return( ret );
	}

	if( strcmp( opt , "ALLOW-IPV6" ) == 0 )
	{
		ret[0] = "-6";
		ret[1] = NULL;
		return( ret );
	}

	if( strcmp( opt , "MIN-RTT" ) == 0 )
	{
		ret[0] = "-min-rtt-timeout";
		ret[1] = calloc( sizeof( char ) , MAXARGSIZE );
		strcpy( ret[1] , arg );
		return( ret );
	}

	if( strcmp( opt , "INITIAL-RTT" ) == 0 )
	{
		ret[0] = "-initial-rtt-timeout";
		ret[1] = calloc( sizeof( char ) , MAXARGSIZE );
		strcpy( ret[1] , arg );
		return( ret );
	}

	if( strcmp( opt , "MAX-RTT" ) == 0 )
	{
		ret[0] = "-max-rtt-timeout";
		ret[1] = calloc( sizeof( char ) , MAXARGSIZE );
		strcpy( ret[1] , arg );
		return( ret );
	}

	if( strcmp( opt , "MIN-RTT" ) == 0 )
	{
		ret[0] = "-min-rtt-timeout";
		ret[1] = calloc( sizeof( char ) , MAXARGSIZE );
		strcpy( ret[1] , arg );
		return( ret );
	}
	if( strcmp( opt , "OSDETECT" ) == 0 )
	{
		ret[0] = "-O";
		ret[1] = NULL;
		return( ret );
	}


	if( strcmp( opt , "TREAT-ONLINE" ) == 0 )
	{
		ret[0] = "-P0";
		ret[1] = NULL;
		return( ret );
	}

	if( strcmp( opt , "VERSION" ) == 0 )
	{
		ret[0] = "-sV";
		ret[1] = NULL;
		return( ret );
	}
	if( strcmp( opt , "OSDETECT-AGRESSIVE" ) == 0 )
	{
		ret[0] = "--osscan-guess";
		ret[1] = NULL;
		return( ret );
	}
	if( strcmp( opt , "PORTS-RANGE" ) == 0 )
	{
		ret[0] = "-p";
		ret[1] = calloc( sizeof( char ) , MAXARGSIZE );
		strncpy( ret[1] , arg , MAXARGSIZE);
		return( ret );
	}
	if( strcmp( opt , "TIMING-TEMPLATE" ) == 0 )
	{
		/* 
		 * With TIMING-TEMPLATE options, arg is always
		 * a number between 0 and 5. So, we are sure that
		 * the arg is on one char
		 */
		ret[0] = calloc( 4 , sizeof( char ) );
		sprintf( ret[0] , "-T%s" , arg );
		ret[1] = NULL;
		return( ret );
	}


	return( NULL );
}


int get_scanid( )
{
	int id;
	id = get_single();
	if( id > 0 )
		return( id );
	id = get_scheduled();
	if( id > 0 )
		return( id );
	return( -1 );
}

/* 
 * Returns a scan-id of a scheduled scan who need one inherited
 * */
int get_scheduled( )
{
	int n;
	time_t ctime;
	char buf[1024];

	PGresult   *res;

	if( time( &ctime ) == -1 )
	{
		return( -1 );
	}

	res = PQexec( dbconn , "BEGIN");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( -1 );
	}
	PQclear( res );


	memset( buf , '\0' , 1024 );
	sprintf( buf , "DECLARE nusers CURSOR FOR select id FROM scans s , scan_opts so WHERE state=%d AND type=%d AND s.id = so.sid AND so.optname='%s' AND so.value<=%li" , STATE_NEW , TYPE_SCHEDULED , DB_SCHEDULEDOPT_STR , (unsigned long) ctime );
	DEBUGMSG( buf );
	res = PQexec( dbconn , buf );
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( -2 );
	}
	PQclear( res );

	res = PQexec( dbconn , "FETCH ALL in nusers");
	if( PQresultStatus( res ) != PGRES_TUPLES_OK )
	{
		PQclear( res );
		return( -3 );
	}
	
	if( PQntuples( res ) <= 0 )
		n = 0;
	else
		n = atoi( PQgetvalue( res , 0 , 0 ));
	PQclear( res );

	res = PQexec( dbconn , "END");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( -1 );
	}
	PQclear( res );

	return( n );
}

int get_single( )
{
	int n;
	char buf[1024];

	PGresult   *res;
	res = PQexec( dbconn , "BEGIN");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( -1 );
	}
	PQclear( res );

	memset( buf , '\0' , 1024 );
	sprintf( buf , "DECLARE nusers CURSOR FOR select id FROM scans WHERE state=%d AND ( type=%d OR type=%d )" , STATE_NEW , TYPE_SINGLE , TYPE_INHERITED );
	DEBUGMSG( buf );
	res = PQexec( dbconn , buf );
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( -2 );
	}
	PQclear( res );

	res = PQexec( dbconn , "FETCH ALL in nusers");
	if( PQresultStatus( res ) != PGRES_TUPLES_OK )
	{
		PQclear( res );
		return( -3 );
	}
	
	if( PQntuples( res ) <= 0 )
		n = 0;
	else
		n = atoi( PQgetvalue( res , 0 , 0 ));
	PQclear( res );
	res = PQexec( dbconn, "END");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( -1 );
	}
    PQclear(res);
	return( n );
}


/* Update the date of the scan_date field in the database
 */
int update_date( const int scanid )
{
	char buf[1024];
	PGresult   *res;

	res = PQexec( dbconn , "BEGIN");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( -1 );
	}
	PQclear( res );

	
	memset( buf , '\0' , 1024 );
	sprintf( buf , "UPDATE scans SET scan_date=NOW() WHERE id=%d" , scanid );
	DEBUGMSG( buf );

	res = PQexec( dbconn , buf );
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		DEBUGMSG( "UPDATE ERROR" );
		PQclear( res );
		return( -1 );
	}
	DEBUGMSG( "UPDATE SUCCESS" );
	PQclear( res );
	res = PQexec( dbconn , "END");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		DEBUGMSG( "UPDATE ERROR" );
		PQclear( res );
		return( -1 );
	}
	PQclear( res );
	DEBUGMSG( "UPDATE SUCCESS" );
	return( 1 );
}


int mark_scan( const int scanid , const int state )
{
	char buf[1024];
	PGresult   *res;

	res = PQexec( dbconn , "BEGIN");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( -1 );
	}
	PQclear( res );

	
	memset( buf , '\0' , 1024 );
	sprintf( buf , "UPDATE scans SET state=%d WHERE id=%d" , state , scanid );
	DEBUGMSG( buf );

	res = PQexec( dbconn , buf );
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		DEBUGMSG( "UPDATE ERROR" );
		PQclear( res );
		return( -1 );
	}
	DEBUGMSG( "UPDATE SUCCESS" );
	PQclear( res );

	res = PQexec( dbconn , "END");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		DEBUGMSG( "UPDATE ERROR" );
		PQclear( res );
		return( -1 );
	}
	PQclear( res );
	DEBUGMSG( "UPDATE SUCCESS" );
	return( 1 );
}

int have_mailtag( const int scanid )
{
	char buf[1024];
	int n;
	PGresult   *res;

	memset( buf , '\0' , 1024 );
	sprintf( buf , "DECLARE myscan CURSOR FOR select * FROM scan_opts WHERE sid=%d AND optname='%s'" , scanid , RIGHT_MAIL );
	res = PQexec( dbconn , "BEGIN");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( 0 );
	}
	PQclear( res );
	DEBUGMSG( buf );

	res = PQexec( dbconn , buf );
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( 0 );
	}
	PQclear( res );

	res = PQexec( dbconn , "FETCH ALL in myscan");
	if( PQresultStatus( res ) != PGRES_TUPLES_OK )
	{
		PQclear( res );
		return( 0 );
	}
	n = PQntuples(res);
	res = PQexec( dbconn , "END");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( 0 );
	}
	PQclear( res );
	return( n );
}

/* Take an scan-id and return the argv table
 * for execution
 */
char** scan2args( int id )
{
	char* optname; 		/* values fetched from database */
	char* optval;		
	char **args; 		/* args we return */
	char **opts;
	char* tmp;
	char buf[1024];
	int i;
	int n;
	PGresult   *res;

	args = calloc( sizeof( char* ) , MAXARGS );

	n = 1;
	args[0] = calloc( sizeof( char ) , MAXARGSIZE );
	strcpy( args[0] , config->cmd );
	
	res = PQexec( dbconn , "BEGIN");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( NULL );
	}
	PQclear( res );

	memset( buf , '\0' , 1024 );
	sprintf( buf , "DECLARE myscan CURSOR FOR select optname , value FROM scan_opts WHERE sid=%d" , id );
	DEBUGMSG( buf );
	res = PQexec( dbconn , buf );
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( NULL );
	}
	PQclear( res );

	res = PQexec( dbconn , "FETCH ALL in myscan");
	if( PQresultStatus( res ) != PGRES_TUPLES_OK )
	{
		PQclear( res );
		return( NULL );
	}

	for( i = 0 ; ( i < PQntuples(res) ) && ( i < ( MAXARGS - 2 )) ; i++ )
	{
		optname = (char*) PQgetvalue( res , i , 0 );
		optval	= (char*) PQgetvalue( res , i , 1 );
		opts = opt2cmdopt( optname , optval );

		if( opts != NULL )
		{
			args[n] = calloc( sizeof( char ) , MAXARGSIZE );
			strcpy( args[n] , opts[0] );
			n++;

			if( opts[1] != NULL )
			{
				args[n] = calloc( sizeof( char ) , MAXARGSIZE );
				strcpy( args[n] , opts[1] );
				free( opts[1] );
				n++;
			}
			free( opts );
		}
	}

	PQclear( res );



	/* Add the option -oX filename ... 
	 * to save the scan result in a file
	 */
	tmp = find_xmlfilename( id );
	if( tmp == NULL )
		return NULL;
	args[n++] = "-oX";
	args[n] = calloc( MAXARGSIZE , sizeof( char ) );
	strcpy( args[n++] , tmp );
	free( tmp );

	tmp = find_txtfilename( id );
	args[n++] = "-oN";
	args[n] = calloc( MAXARGSIZE , sizeof( char ) );
	strcpy( args[n++] , tmp );
	free( tmp );



	/* So, now, add the last argument : the target */
	memset( buf , '\0' , 1024 );
	sprintf( buf , "DECLARE myscan2 CURSOR FOR select target FROM scans WHERE id=%d" , id );
	DEBUGMSG( buf );
	res = PQexec( dbconn , buf );
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( NULL );
	}
	PQclear( res );
	res = PQexec( dbconn , "FETCH ALL in myscan2");
	if( PQresultStatus( res ) != PGRES_TUPLES_OK )
	{
		PQclear( res );
		return( NULL );
	}

	tmp = (char*) PQgetvalue( res , 0 , 0 );
	DEBUGMSG( tmp );


	args[n] = calloc( MAXARGSIZE , sizeof( char ) );
	strcpy( args[n++] , tmp );
	args[n] = NULL;

	PQclear( res );
	res = PQexec( dbconn, "END");
    PQclear(res);
	return( args );
}


char* get_mail( const int scanid )
{
	char buf[1024];
	char* mail;
	char* tmp;
	int n;
	PGresult   *res;

	memset( buf , '\0' , 1024 );
	sprintf( buf , "DECLARE myscan CURSOR FOR select mail from users , scans where scans.uidowner=users.id AND scans.id=%d" , scanid );

	res = PQexec( dbconn , "BEGIN");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( 0 );
	}
	PQclear( res );
	DEBUGMSG( buf );

	res = PQexec( dbconn , buf );
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( 0 );
	}
	PQclear( res );

	res = PQexec( dbconn , "FETCH ALL in myscan");
	if( PQresultStatus( res ) != PGRES_TUPLES_OK )
	{
		PQclear( res );
		return( 0 );
	}

	if( PQntuples(res) != 1 )
	{
		mail = NULL;
	}
	else
	{
		tmp = (char*) PQgetvalue( res , 0 , 0 );
		n = strlen( tmp );
		mail = calloc( n + 1 , sizeof( char ) );
		strncpy( mail , tmp , n );
	}
	res = PQexec( dbconn , "END");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( 0 );
	}
	PQclear( res );
	return( mail );
}

char* get_target( const int scanid )
{
	char buf[1024];
	char* target;
	char *tmp;
	int n;
	PGresult   *res;

	memset( buf , '\0' , 1024 );
	sprintf( buf , "DECLARE myscan CURSOR FOR select target FROM scans WHERE scans.id=%d" , scanid );
	res = PQexec( dbconn , "BEGIN");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( 0 );
	}
	PQclear( res );
	DEBUGMSG( buf );

	res = PQexec( dbconn , buf );
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( 0 );
	}
	PQclear( res );

	res = PQexec( dbconn , "FETCH ALL in myscan");
	if( PQresultStatus( res ) != PGRES_TUPLES_OK )
	{
		PQclear( res );
		return( 0 );
	}

	if( PQntuples(res) != 1 )
	{
		target = NULL;
	}
	else
	{
		tmp = (char*) PQgetvalue( res , 0 , 0 );
		n = strlen( tmp );
		target = calloc( n + 1 , sizeof( char ) );
		strncpy( target , tmp , n );
	}
	res = PQexec( dbconn , "END");
	if( PQresultStatus( res ) != PGRES_COMMAND_OK )
	{
		PQclear( res );
		return( 0 );
	}
	PQclear( res );
	return( target );
}

